package wad.jobs;

import fi.helsinki.cs.tmc.edutestutils.Points;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import static org.assertj.core.api.Assertions.assertThat;
import org.fluentlenium.core.domain.FluentWebElement;
import static org.junit.Assert.assertTrue;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@Points("03-01")
public class JobsTest extends org.fluentlenium.adapter.junit.FluentTest {

    @org.springframework.boot.web.server.LocalServerPort
    private Integer port;

    @Test
    public void canAddJob() {
        addJob();
    }

    @Test
    public void canAddMultipleJobs() {
        addJobs(2);
    }

    @Test
    public void lastJobsAvailableInFirstTable() {
        List<String> jobs = addJobs(7);

        goTo("http://localhost:" + port + "/");

        for (int i = 0; i < jobs.size(); i++) {
            FluentWebElement table = $("table").first();
            String tableText = table.text();

            assertTrue(tableText.contains(jobs.get(i)));
        }
    }

    @Test
    public void canMarkJobDone() {
        String job = markJobDone();

        assertTrue(!$("table").first().text().contains(job));
        assertTrue($("table").get(1).text().contains(job));

        Optional<FluentWebElement> jobRow = $("table").get(1).find("tr").stream().filter((r) -> r.text().contains(job)).findFirst();
        assertTrue(jobRow.isPresent());
        String jobRowText = jobRow.get().text();
        System.out.println(jobRowText);
        assertTrue(jobRowText.contains("Done"));
    }

    public String markJobDone() {
        String job = addJob();
        goTo("http://localhost:" + port + "/");
        FluentWebElement table = $("table").first();
        List<FluentWebElement> elements = table.find("tr").stream().filter((row) -> row.text().contains(job)).collect(Collectors.toList());

        assertTrue("Once a job has been inserted, it should be visible in the first table only once.", elements.size() == 1);

        FluentWebElement e = elements.get(0);
        e.find("form").submit();

        return job;
    }

    public List<String> addJobs(int count) {
        return IntStream.range(0, count).mapToObj(i -> addJob()).collect(Collectors.toList());
    }

    public String addJob() {

        goTo("http://localhost:" + port + "/");

        String job = "Job: " + UUID.randomUUID().toString();

        $("input[name=name]").fill().with(job);
        $("input[value='Add!']").click();

        assertThat(pageSource()).contains(job);

        return job;
    }
}
